import os
import shutil
from pathlib import Path
from PIL import Image
from ultralytics import YOLO
from concurrent.futures import ThreadPoolExecutor
from db_uploader import DatabaseUploader
from config import db_config  # Ensure config.py exists with the correct database configuration


class ImageFilterer:
    def __init__(self, weights_path, image_extensions=None):
        self.model = YOLO(weights_path)
        self.image_extensions = image_extensions or ['*.jpg', '*.jpeg', '*.png', '*.webp']

    def process_image(self, image_path, motorcycle_folder, redundant_folder):
        try:
            result = self.model(Image.open(image_path))[0]
            if result.probs.top1 == 0 and result.probs.top1conf > 0.50:  # Class 0 is 'motorcycle'
                shutil.move(image_path, motorcycle_folder / image_path.name)
            else:
                shutil.move(image_path, redundant_folder / image_path.name)
        except Exception as e:
            print(f"Error processing {image_path}: {e}")

    def process_folder(self, folder_path):
        try:
            motorcycle_folder = folder_path / 'motorcycle'
            redundant_folder = folder_path / 'redundant'

            os.makedirs(motorcycle_folder, exist_ok=True)
            os.makedirs(redundant_folder, exist_ok=True)

            for ext in self.image_extensions:
                for image_path in folder_path.glob(ext):
                    self.process_image(image_path, motorcycle_folder, redundant_folder)

            # Rename the processed folder
            if "__FILTERED" not in folder_path.name:
                processed_folder_name = folder_path.name + "__FILTERED"
                processed_folder_path = folder_path.parent / processed_folder_name
                folder_path.rename(processed_folder_path)
        except Exception as e:
            print(f"Error processing folder {folder_path}: {e}")


def upload_to_db(db_uploader, parent_folder):
    for subfolder in parent_folder.iterdir():
        if subfolder.is_dir() and "__FILTERED" in subfolder.name:
            for image_type in ['motorcycle', 'redundant']:
                folder_path = subfolder / image_type
                for image_path in folder_path.iterdir():
                    file_name = image_path.name
                    file_path = str(image_path.resolve())
                    # Extract additional metadata from the folder name
                    manufacturer, model, year_start, year_end = parse_folder_name(subfolder.name)
                    # Insert metadata directly without checking for existing record
                    db_uploader.insert_metadata(manufacturer, model, year_start, year_end, image_type, file_path,
                                                file_name)


def parse_folder_name(folder_name):
    # Parse the folder name and return the metadata
    # Format of folder should always be: Manufacturer_model_year_year.
    parts = folder_name.replace('__FILTERED', '').split('_')  # Delimiter: '_'
    manufacturer = parts[0]
    model = parts[1]
    year_start = int(parts[2])
    year_end = int(parts[3])
    return manufacturer, model, year_start, year_end


if __name__ == "__main__":
    script_directory = os.path.dirname(os.path.abspath(__file__))

    # Initialize the ImageFilterer
    weights_path = os.path.join(script_directory, 'weights', 'moto_filterer.pt')
    if not os.path.exists(weights_path):
        print("Weights file not found. Please check the weights directory.")
        exit()

    image_filterer = ImageFilterer(weights_path)

    # Ask user for parent folder and number of threads
    parent_folder_path = input("Please enter the path to the parent folder: ")
    parent_folder = Path(parent_folder_path)
    max_workers = int(input("Please specify number of threads: "))

    # Process each folder in parallel
    with ThreadPoolExecutor(max_workers) as executor:
        futures = []
        for subfolder in parent_folder.iterdir():
            if subfolder.is_dir() and "__FILTERED" not in subfolder.name:
                futures.append(executor.submit(image_filterer.process_folder, subfolder))
        for future in futures:
            future.result()  # Wait for all threads to complete

    # After processing all folders, ask the user if he wants to upload to the database
    upload_choice = input("Do you want to upload the filtered data to the database? (y/n): ")
    if upload_choice.lower() == 'y':
        db_uploader = DatabaseUploader(db_config)
        upload_to_db(db_uploader, parent_folder)
        db_uploader.close()

    print("All folders processed.")
