import csv
import os
import time
import multiprocessing
from collections import defaultdict
from urllib.parse import urlparse
from scripts.motohunt_links import MotoHuntLinksScraper
from scripts.motohunt_images import MotoHuntImageScraper
from scripts.autoscout_links import ScoutLinksScraper
from scripts.autoscout_images import ScoutImageScraper
from utils.log_urls import LogInvalidUrls


class ScraperManager:
    # Mapping of domains for scrapers
    SCRAPER_MAPPING = {
        "motohunt.com": {
            "link_scraper": MotoHuntLinksScraper,
            "image_scraper": MotoHuntImageScraper
        },
        "www.autoscout24.com": {
            "link_scraper": ScoutLinksScraper,
            "image_scraper": ScoutImageScraper
        }
    }

    def __init__(self, csv_file, max_concurrent_tasks_per_site):
        self.csv_file = csv_file
        self.max_concurrent_tasks_per_site = max_concurrent_tasks_per_site

        parent_directory = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        self.csv_file = os.path.join(parent_directory, 'data', 'bLinks.csv')

        self.urlLogger = LogInvalidUrls(
            os.path.join(os.path.dirname(os.path.dirname(os.path.abspath(__file__))), 'bLinksLogs'))

    # Check for csv
    def wait_for_csv(self):
        while not os.path.exists(self.csv_file):
            print(f"bLinks.csv not found in 'data' folder. Please add the file to continue... Recheck in 10 seconds")
            time.sleep(10)

    # Read and parse CSV, return dict of sites with their URLs and folder names
    def read_csv(self):
        self.wait_for_csv()
        sites = defaultdict(list)
        invalid_urls = []

        with open(self.csv_file, 'r') as file:
            reader = csv.reader(file, delimiter='|')
            for row in reader:
                if len(row) != 2:
                    invalid_urls.append((' | '.join(row), "Incorrect format: expected URL | FolderName"))
                    print("Incorrect CSV line")
                    continue

                url, folder_name = row[0].strip(), row[1].strip()
                parsed_url = urlparse(url)
                if not parsed_url.scheme or not parsed_url.netloc:
                    invalid_urls.append((url, "Invalid URL"))
                    continue

                domain = parsed_url.netloc
                sites[domain].append((url, folder_name))

        self.urlLogger.log(invalid_urls)
        return sites

    # List of async tasks
    def scraping_for_domain(self, domain, url_pairs):
        print(f"Starting scraping for domain: {domain}")
        # Create a pool of worker processes for concurrent execution
        with multiprocessing.Pool(self.max_concurrent_tasks_per_site) as pool:
            # Async execute scraping tasks for each URL pair in parallel
            results = [pool.apply_async(ScraperWorker.run_scrapers, args=(url_pair,)) for url_pair in url_pairs]
            # Wait for all results to complete before proceeding
            for result in results:
                result.get()

    def start(self):
        sites = self.read_csv()
        processes = []

        for domain, url_pairs in sites.items():
            # Start a separate process for each domain
            p = multiprocessing.Process(target=self.scraping_for_domain, args=(domain, url_pairs))
            processes.append(p)
            p.start()

        for process in processes:
            process.join()


class ScraperWorker:
    @staticmethod
    def run_scrapers(url_folder_pair):
        url, subfolder_name = url_folder_pair
        domain = urlparse(url).netloc
        # Get the corresponding scraper classes for the domain
        scraper_classes = ScraperManager.SCRAPER_MAPPING.get(domain)

        if not scraper_classes:
            print(f"No scraper found for {url}")
            return

        # Initialize the scrapers
        link_scraper = scraper_classes["link_scraper"]()
        image_scraper = scraper_classes["image_scraper"]()

        # Perform scraping
        all_links = link_scraper.scrape_all_links(url)
        print(f"Scraped {len(all_links)} links from {url}")

        # For each link, call the method to extract and download images
        for link in all_links:
            image_scraper.extract_and_download_images(link, subfolder_name)
        print(f"Completed image scraping for {url}")


if __name__ == '__main__':
    csv_file = 'bLinks.csv'
    max_concurrent_tasks_per_site = 3  # Specify if needed
    manager = ScraperManager(csv_file, max_concurrent_tasks_per_site)
    manager.start()
